package server

import (
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/odysseus/service-registry/query"
	"github.com/odysseus/service-registry/registry"
)

var (
	wr registry.Register = workerRegistry{}
)

type workerRegistry struct {
	worker *Worker
	wm     *WorkerManager
}

func (w workerRegistry) ServiceType() common.ServiceType {
	return common.SERVICE_WORKER
}

func (w workerRegistry) Instance() string {
	return fmt.Sprintf("%s", w.worker.addr)
}

func (w workerRegistry) Status() string {
	return fmt.Sprintf("%s", w.worker.status)
}

func (w workerRegistry) DetailInfo() (json.RawMessage, error) {
	if w.worker == nil {
		return nil, fmt.Errorf("worker is nil")
	}
	if w.worker.addr == "" {
		return nil, fmt.Errorf("worker address is empty")
	}

	info := query.WorkerInfo{}
	info.BenefitAddress = w.worker.benefitAddr
	if w.worker.deviceInfo != nil {
		info.IP = w.worker.deviceInfo.DeviceIps[0]
	}
	info.ActiveNM, _ = w.wm.WorkerNmList(w.worker)
	info.HearBeat = w.wm.GetHeartBeat(w.worker.uuid)
	info.MinerAddress = w.worker.addr
	return json.Marshal(info)
}
