package server

import (
	"context"
	"encoding/hex"
	"fmt"
	"github.com/odysseus/nodemanager/config"
	log "github.com/sirupsen/logrus"
	"strconv"
)

func (wm *WorkerManager) UpdateWorkerUsageInfo(worker *Worker, usageInfo string) {
	wm.rdb.Set(context.Background(), workerUsageInfoKey(worker), usageInfo, 0)
}

func (wm *WorkerManager) UpdateWorkerDeviceInfo(worker *Worker, deviceInfos string) {
	wm.rdb.Set(context.Background(), workerDeviceInfoKey(worker), deviceInfos, 0)
}

func (wm *WorkerManager) UpdateWorkerResourceInfo(worker *Worker, resourceInfo []byte) {
	rstr := hex.EncodeToString(resourceInfo)
	wm.rdb.Set(context.Background(), workerResourceInfoKey(worker), rstr, 0)
}

func (wm *WorkerManager) AddWorkerFirst(worker *Worker) error {
	log.WithField("worker", worker.addr).Info("add worker first time.")
	for _, device := range worker.deviceInfo.Devices {
		// add device to redis
		priority := 0
		_ = device // todo: set priority with device info.
		if err := wm.rdb.RPush(context.Background(), config.WORKER_QUEUE_PREFIX+strconv.Itoa(priority), workerId(worker)).Err(); err != nil {
			continue
		}
	}
	// add worker to redis queue
	wm.ActiveWorker(worker)

	return nil
}

func (wm *WorkerManager) AddWorkerSingle(worker *Worker) error {
	log.WithField("worker", worker.addr).Info("add worker on back.")
	{
		// add worker to redis queue
		priority := 0
		if err := wm.rdb.RPush(context.Background(), config.WORKER_QUEUE_PREFIX+strconv.Itoa(priority), workerId(worker)).Err(); err != nil {
			log.WithError(err).Error("add worker back to queue failed.")
		}
	}
	// add worker to redis queue
	wm.ActiveWorker(worker)
	return nil
}

func (wm *WorkerManager) ActiveWorker(worker *Worker) {
	wm.rdb.SAdd(context.Background(), workerStatusKey(worker), config.GetConfig().PublicEndpoint())
}

func (wm *WorkerManager) WorkerNmList(worker *Worker) ([]string, error) {
	return wm.rdb.SMembers(context.Background(), workerStatusKey(worker)).Result()
}

func (wm *WorkerManager) InActiveWorker(worker *Worker) {
	wm.rdb.SRem(context.Background(), workerStatusKey(worker), config.GetConfig().PublicEndpoint())
	if list, err := wm.rdb.SMembers(context.Background(), workerStatusKey(worker)).Result(); err == nil && len(list) == 0 {
		wm.rdb.Del(context.Background(), workerStatusKey(worker))
		wm.rdb.Del(context.Background(), workerUsageInfoKey(worker))
		wm.rdb.Del(context.Background(), workerDeviceInfoKey(worker))
		wm.rdb.Del(context.Background(), workerResourceInfoKey(worker))
	}
}

func workerResourceInfoKey(w *Worker) string {
	return config.WORKER_RESOURCE_INFO_PREFIX + w.addr
}

func workerDeviceInfoKey(w *Worker) string {
	return config.WORKER_DEVICE_INFO_PREFIX + w.addr
}

func workerUsageInfoKey(w *Worker) string {
	return config.WORKER_USAGE_INFO_PREFIX + w.addr
}

func workerStatusKey(w *Worker) string {
	id := workerId(w)
	return fmt.Sprintf("%s_%s", config.WORKER_STATUS_PREFIX, id)
}

func workerId(w *Worker) string {
	return fmt.Sprintf("%s_%d", w.addr, w.uuid)
}
